"""
You do not need to modify read or understand this file.
This file acts as the main thing that runs the code you write in this assignment
so that you can analyze the dataset we provide to you.
"""

from sys import argv
import food_recommender

def main():
    welcome_msg = (
        'Hello! Welcome to the CIS 1100 Restaurant Recommender!\n'
        'Please type in a command to filter on different places to eat!\n'
        'These filters wil apply to each other until you \'reset\' them.\n\n'
        'You can type \'exit\' to exit or \'help\' for a list of commands')

    help_message = (
        '\t help - redisplay this help message\n'
        '\t list - list the current list of restaurants being considered\n'
        '\t reset - resets the list of restaurants being considered based'
        ' on previous criteria and prints the welcome message again\n'
        '\t dist <N> - filters out any restaurants further than the specified'
        ' distance \'N\'\n'
        '\t TA <N> - keep only restaurants that have at least N TA'
        ' endorsements\n'
        '\t cuisine - lists out all possible cuisine options.\n'
        '\t cuisine <specify> - keeps only restaurants of the specified cuisine\n'
        '\t top - keep only the most recommended restaurant of every cuisine\n'
        '\t most - prints which cuisine from the current set has the most'
        ' recommendations out of the current set\n'
        '\t exit - exits the program')

    print(welcome_msg)
    
    if (len(argv) < 2):
        print('ERROR: expected restaurant csv file name as command line arg')

    restaurants = food_recommender.load_restaurant_csv(argv[1])
    print('\nInitial Restaurants:')
    print(restaurants)
    
    current_set = restaurants.keys()
    
    while True:
        command = input('> ')
        if command.startswith('h'):
            print(help_message)
        elif command == 'list':
            print(current_set)
        elif command == 'reset':
            current_set = restaurants.keys()
        elif command.startswith('dist '):
            tokens = command.split()
            if len(tokens) != 2:
                print("invalid number of args, expected 2 for 'dist <N>'")
                continue
            
            try:
                dist = float(tokens[1])
            except:
                print("<N> must be a number. e.g. 3.14")
                continue

            next_set = food_recommender.max_distance(restaurants, dist)

            current_set = current_set & next_set

        elif command.startswith('TA '):
            tokens = command.split()
            if len(tokens) != 2:
                print("invalid number of args, expected 2 for 'TA <N>'")
                continue
            
            try:
                min_rating = int(tokens[1])
            except:
                print("<N> must be a number. e.g. 4")
                continue

            next_set = food_recommender.ta_endorsements(restaurants, min_rating)
            current_set = current_set & next_set

        elif command.startswith('cuisine'):
            tokens = command.split()
            if len(tokens) == 1:
                current_dict = { k:v for k,v in restaurants.items() if k in current_set }
                available_cuisines = food_recommender.get_cuisines(current_dict)
                print(available_cuisines)
                continue
            elif len(tokens) > 2: 
                print("invalid number of args, expected 1 or 2 args for 'cuisines'")
                continue

            cuisine = tokens[1]
            if cuisine not in food_recommender.get_cuisines(restaurants):
                print("\"{}\" is not a cuisine in the CSV".format(cuisine))
                continue

            next_set = food_recommender.filter_cuisine(restaurants, cuisine)
            current_set = current_set & next_set
        elif command.startswith('top'):
            current_dict = { k:v for k,v in restaurants.items() if k in current_set }
            current_set = food_recommender.most_recommended_diverse(current_dict) 
        elif command.startswith('most'):
            current_dict = { k:v for k,v in restaurants.items() if k in current_set }
            print(food_recommender.most_recommended_cuisine(current_dict))
        elif command == "exit":
            break
        else:
            print(f"Unidentified command '{command}'")
            print("Type 'help' for a list of supported commands")

if __name__ == '__main__':
    main()
