# Import Student code
import unittest
import food_recommender

class TestLoadCSV(unittest.TestCase):
    def test_checkpoint_0_loading_empty_dataset(self):
        """Checkpoint 0: Testing load_restaurant_csv() with a CSV file that contains no restaurant info"""
        filename = 'test_files/empty.csv'
        expected = {}
        actual = food_recommender.load_restaurant_csv(filename)
        self.assertEqual(actual, expected)


    def test_checkpoint_0_loading_example(self):
        """Checkpoint 0: Testing load_restaurant_csv() with the spec example file"""
        filename = 'test_files/spec_example.csv'
        expected = {
            "Masala Kitchen": ("Indian", 1, 0.8, 15),
            "Hangry Joes": ("Chicken", 1, 0.4, 12),
        }
        actual = food_recommender.load_restaurant_csv(filename)
        self.assertEqual(actual, expected)


    def test_checkpoint_0_loading_awful_restaurants(self):
        """Checkpoint 0: Testing load_restaurant_csv() with a suspiciously poorly rated restaurant"""
        filename = 'test_files/awful.csv'
        expected = {
            "Hangry Joes": ("Chicken", 1, 0.4, 12),
            "Wawa": ("Fast Food", 1, 0.1, -1),
            "Harry and Travis's Chicken Tendie Surprise": ("Deeply Concerning Chicken", 1, 0.005, -8),
        }
        actual = food_recommender.load_restaurant_csv(filename)
        self.assertEqual(actual, expected)


class TestGetCuisines(unittest.TestCase):

    def test_checkpoint_1_getcuisines_empty_dataset(self):
        """Checkpoint 1: Testing get_cuisines() with an empty restaurant dataset"""
        test_input = {}
        expected = set()
        actual = food_recommender.get_cuisines(test_input)
        self.assertEqual(actual, expected)


    def test_checkpoint_1_getcuisines_example(self):
        """Checkpoint 1: Testing get_cuisines() with the spec example dataset"""
        test_input = {
            "Masala Kitchen": ("Indian", 1, 0.8, 15),
            "Hangry Joes": ("Chicken", 1, 0.4, 12),
        }

        expected = {
            "Indian",
            "Chicken",
        }

        actual = food_recommender.get_cuisines(test_input)
        self.assertEqual(actual, expected)


    def test_checkpoint_1_getcuisines_repeated(self):
        """Checkpoint 1: Testing get_cuisines() with repeated cuisine types"""
        test_input = {
            "Masala Kitchen": ("Indian", 1, 0.8, 15),
            "Amma's South Indian Cuisine": ("Indian", 2, 0.5, 9),
            "Hangry Joes": ("Chicken", 1, 0.4, 12),
            "White Dog Cafe": ("American", 3, 0.3, 1),
        }

        expected = {
            "Indian",
            "Chicken",
            "American",
        }

        actual = food_recommender.get_cuisines(test_input)
        self.assertEqual(actual, expected)



class TestMaxDistance(unittest.TestCase):

    def test_checkpoint_2_maxdistance_empty_dataset(self):
        """Checkpoint 2: Testing max_distance() with an empty restaurant dataset"""
        for i in [-1, 0, 1, 100]:
            test_input = {}
            actual = food_recommender.max_distance(test_input, i)
            expected = set()
            self.assertEqual(actual, expected)


    def test_checkpoint_2_maxdistance_example(self):
        """Checkpoint 2: Testing max_distance() with the spec example dataset"""
        test_input = {
            "Masala Kitchen": ("Indian", 1, 0.8, 15),
            "Hangry Joes": ("Chicken", 1, 0.4, 12),
        }

        expected = set()
        actual = food_recommender.max_distance(test_input, -1)
        self.assertEqual(actual, expected)
        
        expected = set()
        actual = food_recommender.max_distance(test_input, 0)
        self.assertEqual(actual, expected)
        
        expected = {
            "Hangry Joes",
        }
        actual = food_recommender.max_distance(test_input, 0.4)
        self.assertEqual(actual, expected)
        
        expected = {
            "Hangry Joes",
            "Masala Kitchen",
        }
        actual = food_recommender.max_distance(test_input, 0.8)
        self.assertEqual(actual, expected)
        
        expected = {
            "Hangry Joes",
            "Masala Kitchen",
        }
        actual = food_recommender.max_distance(test_input, 1)
        self.assertEqual(actual, expected)


class TestTAEndorsements(unittest.TestCase):

    def test_checkpoint_3_endorsements_empty_dataset(self):
        """Checkpoint 3: Testing ta_endorsements() with an empty restaurant dataset"""
        for i in [100, 1, 0 -1]:
            test_input = {}
            actual = food_recommender.ta_endorsements(test_input, i)
            expected = set()
            self.assertEqual(actual, expected)



    def test_checkpoint_3_endorsements_example(self):
        """Checkpoint 3: Testing ta_endorsements() with the spec example dataset"""
        test_input = {
            "Masala Kitchen": ("Indian", 1, 0.8, 15),
            "Hangry Joes": ("Chicken", 1, 0.4, 12),
        }

        
        actual = food_recommender.ta_endorsements(test_input, 100)
        expected = set()
        self.assertEqual(actual, expected)

        actual = food_recommender.ta_endorsements(test_input, 15)
        expected = {
            "Masala Kitchen",
        }
        self.assertEqual(actual, expected)
        
        actual = food_recommender.ta_endorsements(test_input, 12)
        expected = {
            "Hangry Joes",
            "Masala Kitchen",
        }
        self.assertEqual(actual, expected)
        
        actual = food_recommender.ta_endorsements(test_input, 0)
        expected = {
            "Hangry Joes",
            "Masala Kitchen",
        }
        self.assertEqual(actual, expected)
        
        actual = food_recommender.ta_endorsements(test_input, -100)
        expected = {
            "Hangry Joes",
            "Masala Kitchen",
        }
        self.assertEqual(actual, expected)


class TestFilterCuisine(unittest.TestCase):

    def test_checkpoint_4_filtercuisine_empty_dataset(self):
        """Checkpoint 4: Testing filter_cuisine() with an empty restaurant dataset"""
        for cuisine in ["", "Indian", "anything?", "HELP"]:
            test_input = {}
            actual = food_recommender.filter_cuisine(test_input, cuisine)
            expected = set()
            self.assertEqual(actual, expected)


    def test_checkpoint_4_filtercuisine_example(self):
        """Checkpoint 4: Testing filter_cuisine() with the spec example dataset"""
        test_input = {
            "Masala Kitchen": ("Indian", 1, 0.8, 15),
            "Hangry Joes": ("Chicken", 1, 0.4, 12),
        }

        cuisine = "Indian" 
        expected = {
            "Masala Kitchen",
        }
        actual = food_recommender.filter_cuisine(test_input, cuisine)
        self.assertTrue(actual, expected)
        
        cuisine = "Chicken"
        expected = {
            "Hangry Joes",
        }
        actual = food_recommender.filter_cuisine(test_input, cuisine)
        self.assertTrue(actual, expected)
        
        cuisine = "None"
        expected = set()
        actual = food_recommender.filter_cuisine(test_input, cuisine)
        self.assertEqual(actual, expected)
        
        cuisine = "Indian or Chicken"
        expected = set()
        actual = food_recommender.filter_cuisine(test_input, cuisine)
        self.assertEqual(actual, expected)


class TestMostRecommendedDiverse(unittest.TestCase):

    def test_checkpoint_5_mostreccdiverse_empty_dataset(self):
        """Checkpoint 5: Testing most_recommended_diverse() with an empty restaurant dataset"""
        test_input = {}
        expected = set()
        actual = food_recommender.most_recommended_diverse(test_input)
        self.assertEqual(actual, expected)


    def test_checkpoint_5_mostreccdiverse_example(self):
        """Checkpoint 5: Testing most_recommended_diverse() with the spec example dataset"""
        test_input = {
            "Masala Kitchen": ("Indian", 1, 0.8, 15),
            "Hangry Joes": ("Chicken", 1, 0.4, 12),
        }

        expected =  {
                "Masala Kitchen",
                "Hangry Joes",
        }
        actual = food_recommender.most_recommended_diverse(test_input)
        self.assertEqual(actual, expected)

class TestMostRecommendedAverageCuisine(unittest.TestCase):
    def test_checkpoint_6_single_category_test(self):
        """
        Checkpoint 6: Testing most_recommended_cuisine with a dataset where
        the highest is a restaurant in its own category
        """
        test_input = {
            "Masala Kitchen": ("Indian", 1, 0.8, 15),
            "Amma's South Indian Cuisine": ("Indian", 2, 0.5, 9),
            "Hangry Joes": ("Chicken", 1, 0.4, 12),
            "Cleavers": ("Cheesesteak", 2, 1.25, 14),
            "White Dog Cafe": ("American", 3, 0.3, 1),
            "Where is my money going?": ("Cafe?", 12, 0.5, 0),
            "Wawa": ("Fast Food", 1, 0.1, -1),
            "The Other Wawa": ("Fast Food", 1, 0.2, -5),
            "The Third Wawa": ("Slow Food", 1, 0.1, 0),
            "Harry and Travis's Chicken Tendie Surprise": ("Deeply Concerning Chicken", 1, 0.005, -8),
            "Dick's": ("Fast Food", 1, 2791, 3),
            "Shake Shack": ("Fast Food", 2, 0.2, 3),
            "Levine Hall Snacks": ("Snacks", 2, 0, 15),
        }

        expected = "Snacks"
        actual = food_recommender.most_recommended_cuisine(test_input)
        self.assertEqual(actual, expected)

if __name__ == '__main__':
    unittest.main()
